@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <config.h>
@@ -285,27 +285,21 @@ void
 ev_print_operation_set_embed_page_setup (EvPrintOperation *op,
 					 gboolean          embed)
 {
-#if GTK_CHECK_VERSION (2, 17, 4)
 	EvPrintOperationClass *class = EV_PRINT_OPERATION_GET_CLASS (op);
 
 	g_return_if_fail (EV_IS_PRINT_OPERATION (op));
 
 	class->set_embed_page_setup (op, embed);
-#endif
 }
 
 gboolean
 ev_print_operation_get_embed_page_setup (EvPrintOperation *op)
 {
-#if GTK_CHECK_VERSION (2, 17, 4)
 	EvPrintOperationClass *class = EV_PRINT_OPERATION_GET_CLASS (op);
 
 	g_return_val_if_fail (EV_IS_PRINT_OPERATION (op), FALSE);
 
 	return class->get_embed_page_setup (op);
-#else
-	return FALSE;
-#endif
 }
 
 const gchar *
@@ -324,7 +318,6 @@ ev_print_operation_get_progress (EvPrintOperation *op)
 	return op->progress;
 }
 
-#if GTK_CHECK_VERSION (2, 17, 1) | GTKUNIXPRINT_ENABLED
 static void
 ev_print_operation_update_status (EvPrintOperation *op,
 				  gint              page,
@@ -338,11 +331,11 @@ ev_print_operation_update_status (EvPrintOperation *op,
 
 	if (page == -1) {
 		/* Initial state */
-		op->status = g_strdup (_("Preparing to print ..."));
+		op->status = g_strdup (_("Preparing to print…"));
 	} else if (page > n_pages) {
-		op->status = g_strdup (_("Finishing..."));
+		op->status = g_strdup (_("Finishing…"));
 	} else {
-		op->status = g_strdup_printf (_("Printing page %d of %d..."),
+		op->status = g_strdup_printf (_("Printing page %d of %d…"),
 					      page, n_pages);
 	}
 
@@ -350,7 +343,6 @@ ev_print_operation_update_status (EvPrintOperation *op,
 
 	g_signal_emit (op, signals[STATUS_CHANGED], 0);
 }
-#endif
 
 #if GTKUNIXPRINT_ENABLED
 
@@ -1234,7 +1226,7 @@ ev_print_operation_export_print_dialog_response_cb (GtkDialog              *dial
 						 "%s", _("Invalid page selection"));
 		gtk_window_set_title (GTK_WINDOW (message_dialog), _("Warning"));
 		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message_dialog),
-							  "%s", _("Your print range selection does not include any page"));
+							  "%s", _("Your print range selection does not include any pages"));
 		g_signal_connect (message_dialog, "response",
 				  G_CALLBACK (gtk_widget_destroy),
 				  NULL);
@@ -1319,10 +1311,9 @@ ev_print_operation_export_run (EvPrintOperation *op,
 		ev_file_exporter_get_capabilities (EV_FILE_EXPORTER (op->document));
 	gtk_print_unix_dialog_set_manual_capabilities (GTK_PRINT_UNIX_DIALOG (dialog),
 						       capabilities);
-#if GTK_CHECK_VERSION (2, 17, 4)
+
 	gtk_print_unix_dialog_set_embed_page_setup (GTK_PRINT_UNIX_DIALOG (dialog),
 						    export->embed_page_setup);
-#endif
 
 	gtk_print_unix_dialog_set_current_page (GTK_PRINT_UNIX_DIALOG (dialog),
 						export->current_page);
@@ -1368,11 +1359,9 @@ static void
 ev_print_operation_export_set_embed_page_setup (EvPrintOperation *op,
 						gboolean          embed)
 {
-#if GTK_CHECK_VERSION (2, 17, 4)
 	EvPrintOperationExport *export = EV_PRINT_OPERATION_EXPORT (op);
 
 	export->embed_page_setup = embed;
-#endif
 }
 
 static gboolean
@@ -1502,7 +1491,6 @@ ev_print_operation_export_class_init (EvPrintOperationExportClass *klass)
 
 #endif /* GTKUNIXPRINT_ENABLED */
 
-#if GTK_CHECK_VERSION (2, 17, 1)
 /* Print to cairo interface */
 #define EV_TYPE_PRINT_OPERATION_PRINT         (ev_print_operation_print_get_type())
 #define EV_PRINT_OPERATION_PRINT(object)      (G_TYPE_CHECK_INSTANCE_CAST((object), EV_TYPE_PRINT_OPERATION_PRINT, EvPrintOperationPrint))
@@ -1514,6 +1502,16 @@ typedef struct _EvPrintOperationPrintClass EvPrintOperationPrintClass;
 
 static GType ev_print_operation_print_get_type (void) G_GNUC_CONST;
 
+typedef enum {
+	EV_SCALE_NONE,
+	EV_SCALE_SHRINK_TO_PRINTABLE_AREA,
+	EV_SCALE_FIT_TO_PRINTABLE_AREA
+} EvPrintScale;
+
+#define EV_PRINT_SETTING_PAGE_SCALE "evince-print-setting-page-scale"
+#define EV_PRINT_SETTING_AUTOROTATE "evince-print-setting-page-autorotate"
+#define EV_PRINT_SETTING_PAGE_SIZE  "evince-print-setting-page-size"
+
 struct _EvPrintOperationPrint {
 	EvPrintOperation parent;
 
@@ -1522,6 +1520,14 @@ struct _EvPrintOperationPrint {
 	gint               total;
 	EvJob             *job_print;
 	gchar             *job_name;
+
+        /* Page handling tab */
+        GtkWidget   *scale_combo;
+        EvPrintScale page_scale;
+	GtkWidget   *autorotate_button;
+	gboolean     autorotate;
+	GtkWidget   *source_button;
+	gboolean     use_source_size;
 };
 
 struct _EvPrintOperationPrintClass {
@@ -1632,23 +1638,17 @@ static void
 ev_print_operation_print_set_embed_page_setup (EvPrintOperation *op,
 					       gboolean          embed)
 {
-#if GTK_CHECK_VERSION (2, 17, 4)
 	EvPrintOperationPrint *print = EV_PRINT_OPERATION_PRINT (op);
 
 	gtk_print_operation_set_embed_page_setup (print->op, embed);
-#endif
 }
 
 static gboolean
 ev_print_operation_print_get_embed_page_setup (EvPrintOperation *op)
 {
-#if GTK_CHECK_VERSION (2, 17, 4)
 	EvPrintOperationPrint *print = EV_PRINT_OPERATION_PRINT (op);
 
 	return gtk_print_operation_get_embed_page_setup (print->op);
-#else
-	return FALSE;
-#endif
 }
 
 static void
@@ -1679,13 +1679,11 @@ ev_print_operation_print_done (EvPrintOperationPrint  *print,
 static void
 ev_print_operation_print_status_changed (EvPrintOperationPrint *print)
 {
-#ifdef HAVE_GTK_PRINT_OPERATION_GET_N_PAGES_TO_PRINT
 	GtkPrintStatus status;
 
 	status = gtk_print_operation_get_status (print->op);
 	if (status == GTK_PRINT_STATUS_GENERATING_DATA)
 		print->n_pages_to_print = gtk_print_operation_get_n_pages_to_print (print->op);
-#endif
 }
 
 static void
@@ -1695,12 +1693,11 @@ print_job_finished (EvJobPrint            *job,
 	EvPrintOperation *op = EV_PRINT_OPERATION (print);
 
 	gtk_print_operation_draw_page_finish (print->op);
-#ifdef HAVE_GTK_PRINT_OPERATION_GET_N_PAGES_TO_PRINT
+
 	print->total++;
 	ev_print_operation_update_status (op, print->total,
 					  print->n_pages_to_print,
 					  print->total / (gdouble)print->n_pages_to_print);
-#endif
 	ev_job_print_set_cairo (job, NULL);
 }
 
@@ -1719,13 +1716,39 @@ ev_print_operation_print_request_page_setup (EvPrintOperationPrint *print,
 {
 	EvPrintOperation *op = EV_PRINT_OPERATION (print);
 	gdouble           width, height;
+	GtkPaperSize     *paper_size;
 
 	ev_document_get_page_size (op->document, page_nr,
 				   &width, &height);
-	if (width > height)
-	        gtk_page_setup_set_orientation (setup, GTK_PAGE_ORIENTATION_LANDSCAPE);
-	else
-	        gtk_page_setup_set_orientation (setup, GTK_PAGE_ORIENTATION_PORTRAIT);
+
+	if (print->use_source_size) {
+		paper_size = gtk_paper_size_new_custom ("custom", "custom",
+							width, height, GTK_UNIT_POINTS);
+		gtk_page_setup_set_paper_size_and_default_margins (setup, paper_size);
+		gtk_paper_size_free (paper_size);
+	}
+
+	if (print->autorotate) {
+		if (width > height)
+			gtk_page_setup_set_orientation (setup, GTK_PAGE_ORIENTATION_LANDSCAPE);
+		else
+			gtk_page_setup_set_orientation (setup, GTK_PAGE_ORIENTATION_PORTRAIT);
+	}
+}
+
+static void
+_print_context_get_hard_margins (GtkPrintContext *context,
+				 gdouble         *top,
+				 gdouble         *bottom,
+				 gdouble         *left,
+				 gdouble         *right)
+{
+	if (!gtk_print_context_get_hard_margins (context, top, bottom, left, right)) {
+		*top = 0;
+		*bottom = 0;
+		*left = 0;
+		*right = 0;
+	}
 }
 
 static void
@@ -1737,6 +1760,8 @@ ev_print_operation_print_draw_page (EvPrintOperationPrint *print,
 	cairo_t          *cr;
 	gdouble           cr_width, cr_height;
 	gdouble           width, height, scale;
+	gdouble           x_scale, y_scale;
+	gdouble           top, bottom, left, right;
 
 	gtk_print_operation_set_defer_drawing (print->op);
 
@@ -1755,18 +1780,138 @@ ev_print_operation_print_draw_page (EvPrintOperationPrint *print,
 	cr = gtk_print_context_get_cairo_context (context);
 	cr_width = gtk_print_context_get_width (context);
 	cr_height = gtk_print_context_get_height (context);
-	ev_document_get_page_size (op->document, page,
-				   &width, &height);
-	if (cr_width / width < cr_height / height)
-	        scale = cr_width / width;
-	else
-	        scale = cr_height / height;
-	cairo_scale (cr, scale, scale);
+	ev_document_get_page_size (op->document, page, &width, &height);
+
+	if (print->page_scale == EV_SCALE_NONE) {
+		/* Center document page on the printed page */
+		if (print->autorotate)
+			cairo_translate (cr, (cr_width - width) / 2, (cr_height - height) / 2);
+	} else {
+		_print_context_get_hard_margins (context, &top, &bottom, &left, &right);
+
+		x_scale = (cr_width - left - right) / width;
+		y_scale = (cr_height - top - bottom) / height;
+
+		if (x_scale < y_scale)
+			scale = x_scale;
+		else
+			scale = y_scale;
+
+		if (print->autorotate) {
+			double left_right_sides, top_bottom_sides;
+
+			cairo_translate (cr, (cr_width - scale * width) / 2,
+					 (cr_height - scale * height) / 2);
+
+			/* Ensure document page is within the margins. The
+			 * scale guarantees the document will fit in the
+			 * margins so we just need to check each side and
+			 * if it overhangs the margin, translate it to the
+			 * margin. */
+			left_right_sides = (cr_width - width*scale)/2;
+			top_bottom_sides = (cr_height - height*scale)/2;
+			if (left_right_sides < left)
+				cairo_translate (cr, left - left_right_sides, 0);
+
+			if (left_right_sides < right)
+				cairo_translate (cr, -(right - left_right_sides), 0);
+
+			if (top_bottom_sides < top)
+				cairo_translate (cr, 0, top - top_bottom_sides);
+
+			if (top_bottom_sides < bottom)
+				cairo_translate (cr, 0, -(bottom - top_bottom_sides));
+		} else {
+			cairo_translate (cr, left, top);
+		}
+
+		if (print->page_scale == EV_SCALE_FIT_TO_PRINTABLE_AREA || scale < 1.0) {
+			cairo_scale (cr, scale, scale);
+		}
+	}
 
 	ev_job_print_set_cairo (EV_JOB_PRINT (print->job_print), cr);
 	ev_job_scheduler_push_job (print->job_print, EV_JOB_PRIORITY_NONE);
 }
 
+static GObject *
+ev_print_operation_print_create_custom_widget (EvPrintOperationPrint *print,
+					       GtkPrintContext       *context)
+{
+	GtkPrintSettings *settings;
+	GtkWidget        *label;
+	GtkWidget        *table;
+	EvPrintScale      page_scale;
+	gboolean          autorotate;
+	gboolean          use_source_size;
+
+	settings = gtk_print_operation_get_print_settings (print->op);
+	page_scale = gtk_print_settings_get_int_with_default (settings, EV_PRINT_SETTING_PAGE_SCALE, 0);
+	autorotate = gtk_print_settings_has_key (settings, EV_PRINT_SETTING_AUTOROTATE) ?
+		gtk_print_settings_get_bool (settings, EV_PRINT_SETTING_AUTOROTATE) :
+		TRUE;
+	use_source_size = gtk_print_settings_get_bool (settings, EV_PRINT_SETTING_PAGE_SIZE);
+
+	table = gtk_table_new (3, 2, FALSE);
+	gtk_table_set_row_spacings (GTK_TABLE (table), 6);
+	gtk_table_set_col_spacings (GTK_TABLE (table), 12);
+	gtk_container_set_border_width (GTK_CONTAINER (table), 12);
+
+	label =  gtk_label_new (_("Page Scaling:"));
+	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL, 0, 0, 0);
+	gtk_widget_show (label);
+
+	print->scale_combo = gtk_combo_box_new_text ();
+	gtk_combo_box_append_text (GTK_COMBO_BOX (print->scale_combo), _("None"));
+	gtk_combo_box_append_text (GTK_COMBO_BOX (print->scale_combo), _("Shrink to Printable Area"));
+	gtk_combo_box_append_text (GTK_COMBO_BOX (print->scale_combo), _("Fit to Printable Area"));
+	gtk_combo_box_set_active (GTK_COMBO_BOX (print->scale_combo), page_scale);
+	gtk_widget_set_tooltip_text (print->scale_combo,
+		_("Scale document pages to fit the selected printer page. Select from one of the following:\n"
+		  "\n"
+		  "• \"None\": No page scaling is performed.\n"
+		  "\n"
+		  "• \"Shrink to Printable Area\": Document pages larger than the printable area"
+		  " are reduced to fit the printable area of the printer page.\n"
+		  "\n"
+		  "• \"Fit to Printable Area\": Document pages are enlarged or reduced as"
+		  " required to fit the printable area of the printer page.\n"));
+	gtk_table_attach (GTK_TABLE (table), print->scale_combo, 1, 2, 0, 1, GTK_FILL, 0, 0, 0);
+	gtk_widget_show (print->scale_combo);
+
+	print->autorotate_button = gtk_check_button_new_with_label (_("Auto Rotate and Center"));
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (print->autorotate_button), autorotate);
+	gtk_widget_set_tooltip_text (print->autorotate_button,
+		_("Rotate printer page orientation of each page to match orientation of each document page. "
+		  "Document pages will be centered within the printer page."));
+	gtk_table_attach (GTK_TABLE (table), print->autorotate_button, 0, 2, 1, 2, GTK_FILL, 0, 0, 0);
+	gtk_widget_show (print->autorotate_button);
+
+	print->source_button = gtk_check_button_new_with_label (_("Select page size using document page size"));
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (print->source_button), use_source_size);
+	gtk_widget_set_tooltip_text (print->source_button, _("When enabled, each page will be printed on "
+							     "the same size paper as the document page."));
+	gtk_table_attach (GTK_TABLE (table), print->source_button, 0, 2, 2, 3, GTK_FILL, 0, 0, 0);
+	gtk_widget_show (print->source_button);
+
+	return G_OBJECT (table);
+}
+
+static void
+ev_print_operation_print_custom_widget_apply (EvPrintOperationPrint *print,
+					      GtkPrintContext       *context)
+{
+	GtkPrintSettings *settings;
+
+	print->page_scale = gtk_combo_box_get_active (GTK_COMBO_BOX (print->scale_combo));
+	print->autorotate = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (print->autorotate_button));
+	print->use_source_size = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (print->source_button));
+	settings = gtk_print_operation_get_print_settings (print->op);
+	gtk_print_settings_set_int (settings, EV_PRINT_SETTING_PAGE_SCALE, print->page_scale);
+	gtk_print_settings_set_bool (settings, EV_PRINT_SETTING_AUTOROTATE, print->autorotate);
+	gtk_print_settings_set_bool (settings, EV_PRINT_SETTING_PAGE_SIZE, print->use_source_size);
+}
+
 static void
 ev_print_operation_print_finalize (GObject *object)
 {
@@ -1817,7 +1962,16 @@ ev_print_operation_print_init (EvPrintOperationPrint *print)
 	g_signal_connect_swapped (print->op, "request_page_setup",
 				  G_CALLBACK (ev_print_operation_print_request_page_setup),
 				  print);
+	g_signal_connect_swapped (print->op, "create_custom_widget",
+				  G_CALLBACK (ev_print_operation_print_create_custom_widget),
+				  print);
+	g_signal_connect_swapped (print->op, "custom_widget_apply",
+				  G_CALLBACK (ev_print_operation_print_custom_widget_apply),
+				  print);
 	gtk_print_operation_set_allow_async (print->op, TRUE);
+	gtk_print_operation_set_use_full_page (print->op, TRUE);
+	gtk_print_operation_set_unit (print->op, GTK_UNIT_POINTS);
+	gtk_print_operation_set_custom_tab_label (print->op, _("Page Handling"));
 }
 
 static void
@@ -1841,22 +1995,14 @@ ev_print_operation_print_class_init (EvPrintOperationPrintClass *klass)
 
 	g_object_class->finalize = ev_print_operation_print_finalize;
 }
-#endif /* GTK_CHECK_VERSION (2, 17, 1) */
 
-gboolean ev_print_operation_exists_for_document (EvDocument *document)
+gboolean
+ev_print_operation_exists_for_document (EvDocument *document)
 {
 #if GTKUNIXPRINT_ENABLED
-#if GTK_CHECK_VERSION (2, 17, 1)
 	return (EV_IS_FILE_EXPORTER(document) || EV_IS_DOCUMENT_PRINT(document));
 #else
-	return EV_IS_FILE_EXPORTER(document);
-#endif
-#else /* ! GTKUNIXPRINT_ENABLED */
-#if GTK_CHECK_VERSION (2, 17, 1)
 	return EV_IS_DOCUMENT_PRINT(document);
-#else
-	return FALSE;
-#endif
 #endif /* GTKUNIXPRINT_ENABLED */
 }
 
@@ -1868,15 +2014,15 @@ ev_print_operation_new (EvDocument *document)
 
 	g_return_val_if_fail (ev_print_operation_exists_for_document (document), NULL);
 
-#if GTK_CHECK_VERSION (2, 17, 1)
 	if (EV_IS_DOCUMENT_PRINT (document))
 		op = EV_PRINT_OPERATION (g_object_new (EV_TYPE_PRINT_OPERATION_PRINT,
 						       "document", document, NULL));
 	else
-#endif
 #if GTKUNIXPRINT_ENABLED
 		op = EV_PRINT_OPERATION (g_object_new (EV_TYPE_PRINT_OPERATION_EXPORT,
 						       "document", document, NULL));
+#else
+		op = NULL;
 #endif
 	return op;
 }
